<?php

namespace Drupal\editorial_access_manager\Plugin\EntityReferenceSelection;

use Drupal\editorial_access_manager\EditorialAccessManager;
use Drupal\user\Plugin\EntityReferenceSelection\UserSelection;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides selection for assignable users.
 *
 * @EntityReferenceSelection(
 *   id = "editorial_access_manager_assignable_user",
 *   label = @Translation("Assignable user"),
 *   entity_types = {"user"},
 *   group = "editorial_access_manager",
 *   weight = 1
 * )
 */
class AssignableUserSelection extends UserSelection {

  /**
   * Used to get supported entity types.
   *
   * @var \Drupal\editorial_access_manager\EditorialAccessManager
   */
  protected EditorialAccessManager $editorialAccessManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    /** @var \Drupal\editorial_access_manager\EditorialAccessManager $editorial_access_manager */
    $editorial_access_manager = $container->get('editorial_access_manager.manager');
    $instance->editorialAccessManager = $editorial_access_manager;
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getReferenceableEntities($match = NULL, $match_operator = 'CONTAINS', $limit = 0) {
    $configuration = $this->getConfiguration();
    /** @var array $entities */
    $entities = parent::getReferenceableEntities($match, $match_operator, $limit);
    $entities['user'] = array_filter($entities['user'], function (int $uid) use ($configuration) {
      $user = $this->entityTypeManager->getStorage('user')->load($uid);
      if (!$user instanceof UserInterface) {
        return FALSE;
      }
      $user_can_edit_assigned_entity = $this->checkUserCanEditAnyAssignedEntity($user);
      if (!empty($configuration['editorial_content_entity_type'])) {
        $user_can_edit_assigned_entity = $user_can_edit_assigned_entity || $user->hasPermission(sprintf('edit assigned %s', $configuration['editorial_content_entity_type']));
      }
      return $user_can_edit_assigned_entity;
    }, ARRAY_FILTER_USE_KEY);
    return $entities;
  }

  /**
   * Check that the user can edit any of the supported entity types.
   *
   * @param \Drupal\user\UserInterface $user
   *   User.
   *
   * @return bool
   *   TRUE when the user is able to edit at least one entity type.
   */
  protected function checkUserCanEditAnyAssignedEntity(UserInterface $user) {
    if ($user->hasPermission('edit assigned entity')) {
      return TRUE;
    }

    foreach ($this->editorialAccessManager->getSupportedEntityTypesList() as $entity_type_id) {
      if ($user->hasPermission(sprintf('edit assigned %s', $entity_type_id))) {
        return TRUE;
      }
    }

    return FALSE;
  }

}
